package http

import (
	"context"
	"io/ioutil"
	"net/http"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"

	fullenv "xbus.io/xbus/fullenv_tests"
)

func wait(ctx context.Context, t *testing.T, dur time.Duration) {
	t.Helper()

	select {
	case <-time.After(dur):
	case <-ctx.Done():
		require.NoError(t, ctx.Err())
	}
}

func TestFullenv(t *testing.T) {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	defer cancel()

	env, _ := fullenv.NewFullEnvForTest(t, "fullenv", "fullenv_test.yaml")
	defer env.Close()

	require.NoError(t, env.Init())
	require.NoError(t, env.Startup())
	defer func() { assert.NoError(t, env.Shutdown()) }()

	wait(ctx, t, 600*time.Millisecond)

	req, err := http.NewRequestWithContext(ctx, "GET", "http://localhost:8911/test/processrequest", nil)
	require.NoError(t, err)

	resp, err := http.DefaultClient.Do(req)
	require.NoError(t, err)
	defer func() { assert.NoError(t, resp.Body.Close()) }()

	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}

	req, err = http.NewRequestWithContext(ctx, "GET", "http://localhost:8911/test/processrequest?key=test", nil)
	require.NoError(t, err)

	resp, err = http.DefaultClient.Do(req)
	require.NoError(t, err)
	defer func() { assert.NoError(t, resp.Body.Close()) }()

	if resp.StatusCode != 404 {
		t.Fatal("Expected a 404 error")
	}
	data, err := ioutil.ReadAll(resp.Body)
	assert.NoError(t, err)
	assert.Equal(t, `{"error": "nats: Authorization Violation"}`, string(data))
}
