package control

import (
	"context"

	"xbus.io/go-xbus/v3/api/control"
)

func NewOctetStreamWriter(
	ctx context.Context, send func(control.OctetStreamList),
) OctetSteamWriter {
	return OctetSteamWriter{
		ctx:  ctx,
		send: send,
	}
}

type OctetSteamWriter struct {
	send func(control.OctetStreamList)
	ctx  context.Context

	index int64
}

func (w *OctetSteamWriter) Write(data []byte) (int, error) {
	var chunkSize = 1024 * 1024 * 100
	var sentSize int
	for {
		if len(data) == 0 {
			return 0, nil
		}

		// necessary check to avoid slicing beyond
		// slice capacity
		if len(data) < chunkSize {
			chunkSize = len(data)
		}

		chunk := data[0:chunkSize]
		data = data[chunkSize:]

		w.send(control.OctetStreamList{
			Chunks: []control.OctetStreamList_Chunk{{
				Data:  chunk,
				Index: w.index,
			}},
		})
		if w.ctx.Err() != nil {
			return sentSize, w.ctx.Err()
		}
		w.index++
		sentSize += len(chunk)
	}
}
